package java.util;

import java.util.function.DoubleConsumer;
import java.util.function.LongConsumer;
import java.util.function.IntConsumer;
import java.util.function.Consumer;

public interface Spliterator<T>
{
    public static final int ORDERED = 16;
    public static final int DISTINCT = 1;
    public static final int SORTED = 4;
    public static final int SIZED = 64;
    public static final int NONNULL = 256;
    public static final int IMMUTABLE = 1024;
    public static final int CONCURRENT = 4096;
    public static final int SUBSIZED = 16384;
    
    public abstract boolean tryAdvance(Consumer<? super T> p0);
    
    default public void forEachRemaining(Consumer<? super T> consumer) {
        while (this.tryAdvance(consumer)) {}
    }
    
    public abstract Spliterator<T> trySplit();
    
    public abstract long estimateSize();
    
    default public long getExactSizeIfKnown() {
        return ((this.characteristics() & 0x40) == 0x0) ? -1L : this.estimateSize();
    }
    
    public abstract int characteristics();
    
    default public boolean hasCharacteristics(int n) {
        return (this.characteristics() & n) == n;
    }
    
    default public Comparator<? super T> getComparator() {
        throw new IllegalStateException();
    }
    
    public interface OfPrimitive<T, T_CONS, T_SPLITR extends Spliterator.OfPrimitive<T, T_CONS, T_SPLITR>> extends Spliterator<T>
    {
        public abstract T_SPLITR trySplit();
        
        public abstract boolean tryAdvance(T_CONS p0);
        
        default public void forEachRemaining(T_CONS t_CONS) {
            while (this.tryAdvance(t_CONS)) {}
        }
    }
    
    public interface OfInt extends Spliterator.OfPrimitive<Integer, IntConsumer, Spliterator.OfInt>
    {
        public abstract Spliterator.OfInt trySplit();
        
        public abstract boolean tryAdvance(IntConsumer p0);
        
        default public void forEachRemaining(IntConsumer intConsumer) {
            while (this.tryAdvance(intConsumer)) {}
        }
        
        default public boolean tryAdvance(Consumer<? super Integer> consumer) {
            if ((consumer instanceof IntConsumer)) {
                return this.tryAdvance((IntConsumer)consumer);
            }
            if (Tripwire.ENABLED) {
                Tripwire.trip(this.getClass(), "{0} calling Spliterator.OfInt.tryAdvance((IntConsumer) action::accept)");
            }
            return this.tryAdvance(consumer::accept);
        }
        
        default public void forEachRemaining(Consumer<? super Integer> consumer) {
            if ((consumer instanceof IntConsumer)) {
                this.forEachRemaining((IntConsumer)consumer);
            }
            else {
                if (Tripwire.ENABLED) {
                    Tripwire.trip(this.getClass(), "{0} calling Spliterator.OfInt.forEachRemaining((IntConsumer) action::accept)");
                }
                this.forEachRemaining(consumer::accept);
            }
        }
    }
    
    public interface OfLong extends Spliterator.OfPrimitive<Long, LongConsumer, Spliterator.OfLong>
    {
        public abstract Spliterator.OfLong trySplit();
        
        public abstract boolean tryAdvance(LongConsumer p0);
        
        default public void forEachRemaining(LongConsumer longConsumer) {
            while (this.tryAdvance(longConsumer)) {}
        }
        
        default public boolean tryAdvance(Consumer<? super Long> consumer) {
            if ((consumer instanceof LongConsumer)) {
                return this.tryAdvance((LongConsumer)consumer);
            }
            if (Tripwire.ENABLED) {
                Tripwire.trip(this.getClass(), "{0} calling Spliterator.OfLong.tryAdvance((LongConsumer) action::accept)");
            }
            return this.tryAdvance(consumer::accept);
        }
        
        default public void forEachRemaining(Consumer<? super Long> consumer) {
            if ((consumer instanceof LongConsumer)) {
                this.forEachRemaining((LongConsumer)consumer);
            }
            else {
                if (Tripwire.ENABLED) {
                    Tripwire.trip(this.getClass(), "{0} calling Spliterator.OfLong.forEachRemaining((LongConsumer) action::accept)");
                }
                this.forEachRemaining(consumer::accept);
            }
        }
    }
    
    public interface OfDouble extends Spliterator.OfPrimitive<Double, DoubleConsumer, Spliterator.OfDouble>
    {
        public abstract Spliterator.OfDouble trySplit();
        
        public abstract boolean tryAdvance(DoubleConsumer p0);
        
        default public void forEachRemaining(DoubleConsumer doubleConsumer) {
            while (this.tryAdvance(doubleConsumer)) {}
        }
        
        default public boolean tryAdvance(Consumer<? super Double> consumer) {
            if ((consumer instanceof DoubleConsumer)) {
                return this.tryAdvance((DoubleConsumer)consumer);
            }
            if (Tripwire.ENABLED) {
                Tripwire.trip(this.getClass(), "{0} calling Spliterator.OfDouble.tryAdvance((DoubleConsumer) action::accept)");
            }
            return this.tryAdvance(consumer::accept);
        }
        
        default public void forEachRemaining(Consumer<? super Double> consumer) {
            if ((consumer instanceof DoubleConsumer)) {
                this.forEachRemaining((DoubleConsumer)consumer);
            }
            else {
                if (Tripwire.ENABLED) {
                    Tripwire.trip(this.getClass(), "{0} calling Spliterator.OfDouble.forEachRemaining((DoubleConsumer) action::accept)");
                }
                this.forEachRemaining(consumer::accept);
            }
        }
    }
}
